<?xml version="1.0" encoding="utf-8"?>

<!--
Denne transformasjonen viser hvordan man kan generere en .trans-fil fra en .xml


-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:d="http://www.apple.com/DTDs/DictionaryService-1.0.rng"
    xmlns="http://www.w3.org/1999/xhtml"
>
    <!-- Custom function -->
    <xsl:template name="replace-string">
        <xsl:param name="text"/>
        <xsl:param name="replace"/>
        <xsl:param name="with"/>
        <xsl:choose>
            <xsl:when test="contains($text, $replace)">
                <xsl:value-of select="substring-before($text, $replace)"/>
                <xsl:value-of select="$with"/>
                <xsl:call-template name="replace-string">
                    <xsl:with-param name="text" select="substring-after($text, $replace)"/>
                    <xsl:with-param name="replace" select="$replace"/>
                    <xsl:with-param name="with" select="$with"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$text"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- Custom function -->
    <xsl:template name="replace_space_with_underscore">
        <xsl:param name="text"/>
        <xsl:call-template name="replace-string">
            <xsl:with-param name="text" select="$text"/>
            <xsl:with-param name="replace" select="' '"/>
            <xsl:with-param name="with" select="'_'"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:output method="text" version="1.0" encoding="UTF-8"/>
    <xsl:strip-space elements="*"/>


    <xsl:template match="annotationdoc">
        <xsl:text>#!TRANSCRIPTION!#&#10;</xsl:text>
        <xsl:apply-templates />
    </xsl:template>

    <xsl:template match="annotation">
        <xsl:text>"</xsl:text>
        <xsl:value-of select="@id"/>
        <xsl:text>"&#10;</xsl:text>
        <xsl:apply-templates />
        <xsl:text>.&#10;</xsl:text>
    </xsl:template>

    <!-- Part 1 og 2 -->
    <xsl:template match="word[name(*[1]) = 'seg']">
        <xsl:apply-templates select="seg[1]"/><xsl:text>&#9;</xsl:text>
        <xsl:value-of select="@text"/><xsl:text>&#10;</xsl:text>
        <xsl:for-each select="seg[position() > 1]">
            <xsl:apply-templates select="."/><xsl:text>&#10;</xsl:text>
        </xsl:for-each>
    </xsl:template>
    
    <!-- Part 1 og 2 -->
    <xsl:template match="seg">
        <xsl:value-of select="round(@start * 1000)"/>
        <xsl:text>&#9;</xsl:text>
        <xsl:value-of select="round(@end * 1000)"/>
        <xsl:text>&#9;</xsl:text>
        <xsl:value-of select="@label"/>
    </xsl:template>


    <!-- Part 3 -->
    <xsl:template match="complex_word|multiword">
        <!-- Spesiell håndtering av første element -->
        <xsl:apply-templates select="*[1]"/>
        <xsl:text>&#9;</xsl:text>
        <xsl:call-template name="replace_space_with_underscore">
            <xsl:with-param name="text" select="@text"/>
        </xsl:call-template>
        <xsl:if test="@deadend">;deadend=<xsl:value-of select="@deadend"/></xsl:if>
        <xsl:text>;tag=</xsl:text><xsl:value-of select="name()"/>
        <xsl:text>&#10;</xsl:text>
        <!-- Resten -->
        <xsl:for-each select="*[position() > 1]">
            <xsl:apply-templates select="."/><xsl:text>&#10;</xsl:text>
        </xsl:for-each>
    </xsl:template>

    <!-- Part 3 -->
    <xsl:template match="word|slang">
        <!-- Felt 1 og 2 start og slutt-tid. Ikke obligatoriske attributter -->
        <xsl:choose>
            <xsl:when test="@start">
                <xsl:value-of select="round(@start * 1000)"/>
                <xsl:text>&#9;</xsl:text>
                <xsl:value-of select="round(@end * 1000)"/>
                <xsl:text>&#9;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>_&#9;_&#9;</xsl:text>
            </xsl:otherwise>
        </xsl:choose>

        <!-- Felt 3 er ordstrengen:
            - attributten 'deadend' kodes som "*"-prefiks
            - Vi legger til et kolon pluss normalisert form hvis attributten 'normalized' er tilgjengelig
        -->
        <xsl:value-of select="@text"/>
        <xsl:if test="@deadend">;deadend=<xsl:value-of select="@deadend"/></xsl:if>
        <xsl:if test="@normalized">;normalized=<xsl:value-of select="@normalized"/></xsl:if>
        <xsl:if test="@lang">;lang=<xsl:value-of select="@lang"/></xsl:if>
        
        
        <!-- Felt 4 er gruppe-nivå og bare relevant her hvis elementet ikke står i et
        gruppe-element (complex_word|multiword) -->
        <xsl:if test="name(..) = 'annotation'">
            <xsl:text>&#9;</xsl:text>
            <xsl:value-of select="@text"/>
            <xsl:if test="@deadend">;deadend=<xsl:value-of select="@deadend"/></xsl:if>
            <xsl:if test="@normalized">;normalized=<xsl:value-of select="@normalized"/></xsl:if>
            <xsl:if test="@lang">;lang=<xsl:value-of select="@lang"/></xsl:if>
            <xsl:text>&#10;</xsl:text>
        </xsl:if>
    </xsl:template>

    <!-- Part 3 -->
    <xsl:template match="sil|fp|inhale|exhale|nasal|vowel|anonymized|incomprehensible">
        <!-- Felt 1: start-tid -->
        <xsl:value-of select="round(@start * 1000)"/>
        <xsl:text>&#9;</xsl:text>
        <!-- Felt 2: slutt-tid -->
        <xsl:value-of select="round(@end * 1000)"/>
        <xsl:text>&#9;&lt;</xsl:text>

        <!-- Felt 3: navn -->
        <xsl:value-of select="name(.)"/>
        <xsl:text>&gt;</xsl:text>
        <!-- Felt 3: navn, hvis ikke i en gruppe (complex_word|multiword) -->
        <xsl:if test="name(ancestor::*[1]) = 'annotation'">
            <xsl:text>&#9;&lt;</xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text>&gt;</xsl:text>
            <xsl:text>&#10;</xsl:text>
        </xsl:if>
    </xsl:template>

    <!-- Part 3 -->
    <xsl:template match="comma|titlestart|titleend|sentence_boundary">
        <xsl:choose>
            <xsl:when test="@start">
                <xsl:value-of select="round(@start * 1000)"/>
                <xsl:text>&#9;</xsl:text>
                <xsl:value-of select="round(@end * 1000)"/>
                <xsl:text>&#9;</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>_&#9;_&#9;</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
       
        <xsl:text>&lt;</xsl:text>
        <xsl:value-of select="name(.)"/>
        <xsl:text>&gt;&#9;&lt;</xsl:text>
        <xsl:value-of select="name(.)"/>
        <xsl:text>&gt;&#10;</xsl:text>
    </xsl:template>

</xsl:stylesheet>
